#!/usr/bin/env python
import h5py
import numpy as np
from torch.utils.data import Dataset
import random
from transformers import AutoTokenizer

class LoopPredictionDataset(Dataset):
    def __init__(
        self,
        data_file,
        tokenizer=AutoTokenizer.from_pretrained("AIRI-Institute/gena-lm-bert-base-t2t"),
        max_seq_len=512,
    ):
        self.data = h5py.File(data_file, "r")

        self.samples = int(len(self.data.keys())/6)
        self.tokenizer = tokenizer
        self.max_seq_len = max_seq_len

    def __len__(self):
        
        return self.samples
        
    def __getitem__(self, idx):
        assert idx < self.samples
        control_or_not=random.choice(['','control_'])
        key1 = control_or_not + f'pair_{idx}_seq1'
        key2= control_or_not + f'pair_{idx}_seq2'
        key_val= control_or_not + f'pair_{idx}_val'
        seq1=self.data[key1][()].decode().upper()
        seq2=self.data[key2][()].decode().upper()
        seq_tokenized=self.tokenizer(text=seq1,text_pair=seq2, add_special_tokens=True, return_tensors='np', padding=False, truncation=True, max_length=self.max_seq_len)
        label=self.data[key_val][()]

        return {
            "input_ids": seq_tokenized['input_ids'].ravel(),
            "token_type_ids": seq_tokenized['token_type_ids'].ravel(),
            "attention_mask": seq_tokenized['attention_mask'].ravel(),
            "labels": float(label),
        }

    def close(self):
        self.data.close()